# -*- Perl -*-
#
# File  : Logger.pm
#
# Description :
#   This perl module manages the logging of a script.
#   
#   IT IS A SINGLETON
#
#   This means there is only one copy of the object, All calls to new return the same reference.
#
#   It can log to a file or the console or both.
#   It prints the line number, then the error level, then the error itself.


package Logger;

my $module_name = "Logger";

use Exporter();

# The singleton reference
my $logger_singleton_reference = undef;

# this module inherits from the "Exporter" module
@ISA = qw(Exporter);
# functions we're exporting
@EXPORT = qw(   CONSOLE_OFF
                CONSOLE_ON
                CONSOLE_TERSE);

use strict;
use File::Basename qw(basename);
use IO::File;
use Time::localtime;

# Static functions
# Console output state
# Output nothing
sub CONSOLE_OFF() {
    return 0;
}

# Output everything
sub CONSOLE_ON() {
    return 1;
}

# Output Success messages
sub CONSOLE_TERSE() {
    return 2;
}

#*********************************
# PUBLIC functions
#*********************************

#
# Constructor
# 
# Arguments:
# @param 1 - self, the logger object itself.
#
# Returns; 
# a reference to itself
#
sub new {
    my $class = shift;
    
    # Is the $singleton defined yet?
    if (! defined $logger_singleton_reference) {
        
        my $self = {};
        # It's not so bless it
        $logger_singleton_reference = bless $self, $class;

        # define module private data members
        # the console is on by default
        $logger_singleton_reference->{m_consoleState}=CONSOLE_ON;
        $logger_singleton_reference->{m_isFile}=0;
        $logger_singleton_reference->{m_count}=0;
        $logger_singleton_reference->{m_file_handle}=undef;
        $logger_singleton_reference->{m_file_name}="";
    }

    # Return the reference
    return $logger_singleton_reference;
}

#
# Opens a file for printing
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the file name to open
#
# Returns; 
# int: returns 0 for failure, 1 for success
#
sub log_set_file_on () {
    my $self = shift;
    my $file_name = shift;

    return $self->log_prv_open_file($file_name);
}

#
# Typically prints the output of an executable captured by UtilLib::ut_shell
# It splits the $output by line break, prepends the log line, then prints the whole mess.
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the log string
# @param 3 - the output
#
# Returns; 
# nothing
#
sub log_print_output() {
    my $self = shift;
    my $log_prefix = shift;
    my $output = shift;
    
    # no input, return;
    if ( ( ! $log_prefix ) || ( ! $output )) {
        return;
    }
    
    # split the output on line breaks
    my (@out_lines) = split (/\n/,$output);
    
    #print them all out
    foreach (@out_lines) {
        $self->log_print_message($log_prefix . " " . $_);
    }
}

#
# Prints an debug message
# If there's nothing to print, it returns after doing nothing
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the string to print
#
# Returns; 
# nothing
#
sub log_print_debug() {
    my $self = shift;
    my $value = shift;
    $self->log_prv_print('DEBUG', $value);
}

#
# Prints an error.
# If there's nothing to print, it returns after doing nothing
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the string to print
#
# Returns; 
# nothing
#
sub log_print_error() {
    my $self = shift;
    my $value = shift;
    $self->log_prv_print('ERROR', $value);
}

#
# Prints a warning.
# If there's nothing to print, it returns
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the string to print
#
# Returns; 
# nothing
#
sub log_print_warning() {
    my $self = shift;
    my $value = shift;
    $self->log_prv_print('WARNING', $value);
}

#
# Prints a generic message.
# If there's nothing to print, it returns
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the string to print
#
# Returns; 
# nothing
#
sub log_print_message() {
    my $self = shift;
    my $value = shift;
    $self->log_prv_print('MESSAGE', $value);
}

#
# Prints a generic success message.
# If there's nothing to print, it returns
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the string to print
#
# Returns; 
# nothing
#
sub log_print_success() {
    my $self = shift;
    my $value = shift;
    $self->log_prv_print('SUCCESS', $value);
}

#
# Prints a generic milestone message.
# If there's nothing to print, it returns
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the string to print
#
# Returns; 
# nothing
#
sub log_print_milestone() {
    my $self = shift;
    my $value = shift;
    $self->log_prv_print('MILESTONE', $value);
}

#
# Sets the console logging state
# options are CONSOLE_OFF=1 CONSOLE_ON=1 and CONSOLE_TERSE=2
# 
# Arguments:
# @param 1 - self, the logger object itself.
#
# Returns; 
# nothing
#
sub log_set_console() {
    my $self = shift;
    my $value = shift;

    $self->{m_consoleState} = $value;
}

#
# Turns off logging to a file
# 
# Arguments:
# @param 1 - self, the logger object itself.
#
# Returns; 
# nothing
#
sub log_set_file_off() {
    my $self = shift;

    $self->log_prv_close_file();
}

#
# Builds a log summary.
#
# NOTE: This is a very special function. It's called at the end of the build
# when all the resources are closing, including the build log file. That's why
# it doesn't have any logging itself.
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the name of the log file
#
# Returns; 
# 1 if successful, 0 if not
# the summary in a single variable
#
sub log_summary() {
    # NOTE: if this functions being copied to the log_sum.pl script, comment out the following line: my $self = shift;
    my $self = shift;
    my $log_file = shift;

    if (!$log_file) {
        return 0;
    }
    
    if (! open (IN, "<$log_file")) {
        return 0;
    }

    my @errors;
    my @warnings;
    my @urgent; # URGENT wsbu import errors
    my %build_times;
    my $wsbu_time;
    my $total_time;

    while (<IN>) {
        if(/[ :]error[ :]/i) {
            push(@errors, $_);
        } elsif(/[ :]warning[ :]/i) {
            push(@warnings, $_);
        } elsif(/\bURGENT\b/) {
            push(@urgent, $_);
        }
        if(/BUILD_TIME--(.*)--(.*)/) {
            $build_times{$1} = $2;
        }
        if(/WSBU_TIME--(.*)/) {
            $wsbu_time = $1;
        }
        if(/TOTAL_TIME--(.*)/) {
            $total_time = $1;
        }
    }

    close (IN);

    my $summary = "";;

    if (%build_times) {
        $summary = "\n\nrh:\n";
        $summary .= "*******************************************************\n";
        foreach my $key (keys(%build_times)) {
            $summary .= "    $key:$build_times{$key}\n";
        }
    }

    if ($wsbu_time) {
        $summary .= "\n\nWSBU C|[g:\n";
        $summary .= "*******************************************************\n";
        $summary .= "    $wsbu_time\n";
    }
    
    if ($total_time) {
        $summary .= "\n\nv:\n";
        $summary .= "*******************************************************\n";
        $summary .= "    $total_time\n";
    }
    
    if (@urgent) {
        $summary .= "\n\nً} WSBU t@C̃C|[g G[: \n";
        $summary .= "*******************************************************\n";
        $summary .= join('', @urgent);
    }   
    
    $summary .= "\n\nO t@C̃G[: \n";
    $summary .= "*******************************************************\n";
    $summary .= join('', @errors);

    $summary .= "\n\nO t@Čx: \n";
    $summary .= "*******************************************************\n";
    $summary .= join('', @warnings);

    return 1, $summary;
}

#*********************************
# PRIVATE functions
#*********************************


#
# Prints a line to the log
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - the text to print
#
# Returns; 
# nothing
#
    
sub log_prv_print() {
    my $self = shift;
    my $kind = shift;
    my $entry = shift;  # the text to print

    if (!$entry) { return };

    # the line counter
    my $count = $self->{m_count}++;
    # the formatted time
    my $time = $self->log_prv_time();
    # complete output
    $entry = $self->log_prv_add_caller($entry);
    chomp($entry);
    my $out = sprintf("%05d:%s:%-9s: %s\n", $count, $time, $kind, $entry);
    
    if ($self->{m_consoleState} == CONSOLE_ON) {
        print $out;
    } elsif ( ($self->{m_consoleState} == CONSOLE_TERSE) && ( ($kind eq "") or ($kind eq "}CXg[") or ($kind eq "G[") ) ) {
        print $out;
    }

    if ($self->{m_isFile} == 1) {
        # for some reason we have to convert the value in the object's hash
        # to a variable
        my $fn = $self->{m_file_handle};
        print $fn $out;
    }
}

# Add caller info if $entry doesn't have it.
sub log_prv_add_caller($$) {
    my $self = shift;
    my $entry = shift;
    if($entry =~ /^\w+: \w+(\(\))?:/) {
        return $entry;  # already have caller info
    }
    # find the first caller that isn't Logger::log_print_*
    my $prev_filename = undef;
    for(my $i = 2; ; $i += 1) {
        my(undef, $filename, undef, $full_sub) = caller($i);
        if(!defined($full_sub)) {
            return $entry;  # didn't find a suitable caller
        }
        my($package, $sub) = $full_sub =~ /^(.*)::(.*)$/ && ($1, $2);
        if($package eq 'main' && defined($prev_filename)) {
            $package = basename($prev_filename);
            $package =~ s/\.pl$//i;
        }
        if($sub ne '__ANON__'
        && ($package ne __PACKAGE__ || $sub !~ /^log_print_/)) {
            return "$package: $sub(): $entry";
        }
        $prev_filename = $filename;
    }
}

#
# Manage the counter. Increments the counter, and formats
# the value so it prints nicely. Only formats to 99999
# 
# Arguments:
# @param 1 - self, the logger object itself.
#
# Returns; 
# string: formated number.
#
sub log_prv_time() {
    my $self = shift;

    my $sec = localtime->sec();
    my $min = localtime->min();
    my $hour = localtime->hour();
    return sprintf("%02d.%02d.%02d", $hour, $min, $sec);
}

#
# Opens a file for logging, and set's the m_isFile to true
# 
# Arguments:
# @param 1 - self, the logger object itself.
# @param 2 - The name of the file to be opened for logging to.
#
# Returns; 
# int: 0 if the call failed, 1 if the call succeeded
#
sub log_prv_open_file() {
    my $self = shift;
    my $file_name = shift;

    my $function_name = "log_prv_open_file";
    
    # bad file name?
    if (!$file_name) {
        $self->log_print_error ("$module_name: $function_name(): ȃt@C");
        return 0;
    }

    # create a new file handle
    $self->{m_file_handle} = new IO::File;
    
    if (! ($self->{m_file_handle}->open(">$file_name"))) {
        # if the call fails, undef the handle
        undef $self->{m_file_handle};       
        $self->log_print_error ("$module_name: $function_name(): t@CJƂł܂");
        return 0;
    }

    # set the local file flag to true;
    $self->{m_isFile} = 1;

    # save the file name for a later summary
    $self->{m_file_name} = $file_name;
    
    $self->log_print_success ("$module_name: $function_name(): ɃO t@C $file_name J܂");
    # return success
    return 1;
}

#
# Closes a logging file
# 
# Arguments:
# @param 1 - self, the logger object itself.
#
# Returns; 
# nothing
#
sub log_prv_close_file() {
    my $self = shift;
    
    my $file_handle = $self->{m_file_handle};
    if ($self->{m_isFile} == 1) {
        
        if($self->{m_file_handle}) {
            $self->{m_file_handle}->close;
        }
        
        # set the internal flag to false
        $self->{m_isFile} = 0;
        
        my $log_file = $self->{m_file_name};
        
        # Create the log summary
        my ($retval, $summary) = $self->log_summary($log_file);        

        if ( ! $retval) {
            return;
        }
        
        # append the summary to the file
        if ( ! open (OUT, ">>$log_file") ) {
            return;
        }
        
        # print to the log file:
        print OUT $summary;
        # print to the console:
        print $summary;
        
        close (OUT);        
    }
}

#
# Module destructor, closes the open logging file, if open
# 
# Arguments:
# @param 1 - self, the logger object itself.
#
# Returns; 
# nothing
#
sub DESTROY () {
    my $self = shift;

    $self->log_prv_close_file();
}

1;
